@@ -17,6 +17,8 @@ use std::{
     sync::{Arc, Mutex, RwLock},
     thread,
 };
+use zellij_utils::envs;
+use zellij_utils::nix::sys::stat::{umask, Mode};
 use zellij_utils::pane_size::Size;
 use zellij_utils::zellij_tile;
 
@@ -27,6 +29,7 @@ use crate::{
     os_input_output::ServerOsApi,
     pty::{pty_thread_main, Pty, PtyInstruction},
     screen::{screen_thread_main, ScreenInstruction},
+    tab::Output,
     thread_bus::{Bus, ThreadSenders},
     wasm_vm::{wasm_thread_main, PluginInstruction},
 };
@@ -59,11 +62,12 @@ pub(crate) enum ServerInstruction {
         ClientId,
         Option<PluginsConfig>,
     ),
-    Render(Option<String>),
+    Render(Option<Output>),
     UnblockInputThread,
     ClientExit(ClientId),
     RemoveClient(ClientId),
     Error(String),
+    KillSession,
     DetachSession(ClientId),
     AttachClient(ClientAttributes, Options, ClientId),
 }
@@ -77,6 +81,7 @@ impl From<&ServerInstruction> for ServerContext {
             ServerInstruction::ClientExit(..) => ServerContext::ClientExit,
             ServerInstruction::RemoveClient(..) => ServerContext::RemoveClient,
             ServerInstruction::Error(_) => ServerContext::Error,
+            ServerInstruction::KillSession => ServerContext::KillSession,
             ServerInstruction::DetachSession(..) => ServerContext::DetachSession,
             ServerInstruction::AttachClient(..) => ServerContext::AttachClient,
         }
@@ -172,13 +177,17 @@ impl SessionState {
 
 pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
     info!("Starting Zellij server!");
+
+    // preserve the current umask: read current value by setting to another mode, and then restoring it
+    let current_umask = umask(Mode::all());
+    umask(current_umask);
     daemonize::Daemonize::new()
         .working_directory(std::env::current_dir().unwrap())
-        .umask(0o077)
+        .umask(current_umask.bits())
         .start()
         .expect("could not daemonize the server process");
 
-    std::env::set_var(&"ZELLIJ", "0");
+    envs::set_zellij("0".to_string());
 
     let (to_server, server_receiver): ChannelWithContext<ServerInstruction> = channels::bounded(50);
     let to_server = SenderWithContext::new(to_server);
@@ -261,7 +270,6 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     os_input.clone(),
                     to_server.clone(),
                     client_attributes,
-                    session_state.clone(),
                     SessionOptions {
                         opts,
                         layout: layout.clone(),
@@ -289,7 +297,11 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .as_ref()
                         .unwrap()
                         .senders
-                        .send_to_pty(PtyInstruction::NewTab(default_shell.clone(), tab_layout))
+                        .send_to_pty(PtyInstruction::NewTab(
+                            default_shell.clone(),
+                            tab_layout,
+                            client_id,
+                        ))
                         .unwrap()
                 };
 
@@ -317,13 +329,17 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .senders
                     .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                     .unwrap();
+                session_data
+                    .senders
+                    .send_to_screen(ScreenInstruction::AddClient(client_id))
+                    .unwrap();
                 let default_mode = options.default_mode.unwrap_or_default();
                 let mode_info =
                     get_mode_info(default_mode, attrs.palette, session_data.capabilities);
                 let mode = mode_info.mode;
                 session_data
                     .senders
-                    .send_to_screen(ScreenInstruction::ChangeMode(mode_info.clone()))
+                    .send_to_screen(ScreenInstruction::ChangeMode(mode_info.clone(), client_id))
                     .unwrap();
                 session_data
                     .senders
@@ -353,6 +369,16 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
+                    // we only do this inside this if because it means there are still connected
+                    // clients
+                    session_data
+                        .write()
+                        .unwrap()
+                        .as_ref()
+                        .unwrap()
+                        .senders
+                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
+                        .unwrap();
                 }
                 if session_state.read().unwrap().clients.is_empty() {
                     *session_data.write().unwrap() = None;
@@ -370,8 +396,26 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
+                    // we only do this inside this if because it means there are still connected
+                    // clients
+                    session_data
+                        .write()
+                        .unwrap()
+                        .as_ref()
+                        .unwrap()
+                        .senders
+                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
+                        .unwrap();
                 }
             }
+            ServerInstruction::KillSession => {
+                let client_ids = session_state.read().unwrap().client_ids();
+                for client_id in client_ids {
+                    os_input.send_to_client(client_id, ServerToClientMsg::Exit(ExitReason::Normal));
+                    remove_client!(client_id, os_input, session_state);
+                }
+                break;
+            }
             ServerInstruction::DetachSession(client_id) => {
                 os_input.send_to_client(client_id, ServerToClientMsg::Exit(ExitReason::Normal));
                 remove_client!(client_id, os_input, session_state);
@@ -384,6 +428,16 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .senders
                         .send_to_screen(ScreenInstruction::TerminalResize(min_size))
                         .unwrap();
+                    // we only do this inside this if because it means there are still connected
+                    // clients
+                    session_data
+                        .write()
+                        .unwrap()
+                        .as_ref()
+                        .unwrap()
+                        .senders
+                        .send_to_screen(ScreenInstruction::RemoveClient(client_id))
+                        .unwrap();
                 }
             }
             ServerInstruction::Render(mut output) => {
@@ -392,8 +446,13 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 // If `Some(_)`- unwrap it and forward it to the clients to render.
                 // If `None`- Send an exit instruction. This is the case when a user closes the last Tab/Pane.
                 if let Some(op) = output.as_mut() {
-                    for client_id in client_ids {
-                        os_input.send_to_client(client_id, ServerToClientMsg::Render(op.clone()));
+                    for (client_id, client_render_instruction) in
+                        op.client_render_instructions.iter_mut()
+                    {
+                        os_input.send_to_client(
+                            *client_id,
+                            ServerToClientMsg::Render(client_render_instruction.clone()),
+                        );
                     }
                 } else {
                     for client_id in client_ids {
@@ -440,7 +499,6 @@ fn init_session(
     os_input: Box<dyn ServerOsApi>,
     to_server: SenderWithContext<ServerInstruction>,
     client_attributes: ClientAttributes,
-    session_state: Arc<RwLock<SessionState>>,
     options: SessionOptions,
 ) -> SessionMetaData {
     let SessionOptions {
@@ -465,7 +523,7 @@ fn init_session(
     let data_dir = opts.data_dir.unwrap_or_else(get_default_data_dir);
 
     let capabilities = PluginCapabilities {
-        arrow_fonts: config_options.simplified_ui,
+        arrow_fonts: config_options.simplified_ui.unwrap_or_default(),
     };
 
     let default_shell = config_options.default_shell.clone().map(|command| {
@@ -508,13 +566,7 @@ fn init_session(
             let max_panes = opts.max_panes;
 
             move || {
-                screen_thread_main(
-                    screen_bus,
-                    max_panes,
-                    client_attributes,
-                    config_options,
-                    session_state,
-                );
+                screen_thread_main(screen_bus, max_panes, client_attributes, config_options);
             }
         })
         .unwrap();
