@@ -8,8 +8,9 @@ use crate::tab::Pane;
 use crate::ui::pane_boundaries_frame::PaneFrame;
 use crate::wasm_vm::PluginInstruction;
 use zellij_utils::pane_size::Offset;
+use zellij_utils::position::Position;
 use zellij_utils::shared::ansi_len;
-use zellij_utils::zellij_tile::prelude::PaletteColor;
+use zellij_utils::zellij_tile::prelude::{Event, Mouse, PaletteColor};
 use zellij_utils::{
     channels::SenderWithContext,
     pane_size::{Dimension, PaneGeom},
@@ -254,14 +255,53 @@ impl Pane for PluginPane {
         self.geom.y -= count;
         self.should_render = true;
     }
-    fn scroll_up(&mut self, _count: usize) {
-        //unimplemented!()
+    fn scroll_up(&mut self, count: usize) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::ScrollUp(count)),
+            ))
+            .unwrap();
     }
-    fn scroll_down(&mut self, _count: usize) {
-        //unimplemented!()
+    fn scroll_down(&mut self, count: usize) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::ScrollDown(count)),
+            ))
+            .unwrap();
     }
     fn clear_scroll(&mut self) {
-        //unimplemented!()
+        unimplemented!();
+    }
+    fn start_selection(&mut self, start: &Position) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::LeftClick(start.line.0, start.column.0)),
+            ))
+            .unwrap();
+    }
+    fn update_selection(&mut self, position: &Position) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::Hold(position.line.0, position.column.0)),
+            ))
+            .unwrap();
+    }
+    fn end_selection(&mut self, end: Option<&Position>) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::Release(
+                    end.map(|Position { line, column }| (line.0, column.0)),
+                )),
+            ))
+            .unwrap();
+    }
+    fn is_scrolled(&self) -> bool {
+        false
     }
 
     fn active_at(&self) -> Instant {
@@ -287,4 +327,12 @@ impl Pane for PluginPane {
     fn borderless(&self) -> bool {
         self.borderless
     }
+    fn handle_right_click(&mut self, to: &Position) {
+        self.send_plugin_instructions
+            .send(PluginInstruction::Update(
+                Some(self.pid),
+                Event::Mouse(Mouse::RightClick(to.line.0, to.column.0)),
+            ))
+            .unwrap();
+    }
 }
