@@ -1,9 +1,10 @@
-use std::convert::TryFrom;
 use std::fmt::{self, Debug, Display, Formatter};
 use std::ops::{Index, IndexMut};
-use zellij_utils::logging::debug_log_to_file;
+
 use zellij_utils::vte::ParamsIter;
 
+use crate::panes::alacritty_functions::parse_sgr_color;
+
 pub const EMPTY_TERMINAL_CHARACTER: TerminalCharacter = TerminalCharacter {
     character: ' ',
     width: 1,
@@ -191,6 +192,7 @@ impl CharacterStyles {
     pub fn update_and_return_diff(
         &mut self,
         new_styles: &CharacterStyles,
+        changed_colors: Option<[Option<AnsiCode>; 256]>,
     ) -> Option<CharacterStyles> {
         let mut diff: Option<CharacterStyles> = None;
 
@@ -223,100 +225,102 @@ impl CharacterStyles {
         if self.foreground != new_styles.foreground {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.foreground(new_styles.foreground));
-                self.foreground = new_styles.foreground;
             } else {
                 diff = Some(CharacterStyles::new().foreground(new_styles.foreground));
-                self.foreground = new_styles.foreground;
             }
+            self.foreground = new_styles.foreground;
         }
         if self.background != new_styles.background {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.background(new_styles.background));
-                self.background = new_styles.background;
             } else {
                 diff = Some(CharacterStyles::new().background(new_styles.background));
-                self.background = new_styles.background;
             }
+            self.background = new_styles.background;
         }
         if self.strike != new_styles.strike {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.strike(new_styles.strike));
-                self.strike = new_styles.strike;
             } else {
                 diff = Some(CharacterStyles::new().strike(new_styles.strike));
-                self.strike = new_styles.strike;
             }
+            self.strike = new_styles.strike;
         }
         if self.hidden != new_styles.hidden {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.hidden(new_styles.hidden));
-                self.hidden = new_styles.hidden;
             } else {
                 diff = Some(CharacterStyles::new().hidden(new_styles.hidden));
-                self.hidden = new_styles.hidden;
             }
+            self.hidden = new_styles.hidden;
         }
         if self.reverse != new_styles.reverse {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.reverse(new_styles.reverse));
-                self.reverse = new_styles.reverse;
             } else {
                 diff = Some(CharacterStyles::new().reverse(new_styles.reverse));
-                self.reverse = new_styles.reverse;
             }
+            self.reverse = new_styles.reverse;
         }
         if self.slow_blink != new_styles.slow_blink {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.blink_slow(new_styles.slow_blink));
-                self.slow_blink = new_styles.slow_blink;
             } else {
                 diff = Some(CharacterStyles::new().blink_slow(new_styles.slow_blink));
-                self.slow_blink = new_styles.slow_blink;
             }
+            self.slow_blink = new_styles.slow_blink;
         }
         if self.fast_blink != new_styles.fast_blink {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.blink_fast(new_styles.fast_blink));
-                self.fast_blink = new_styles.fast_blink;
             } else {
                 diff = Some(CharacterStyles::new().blink_fast(new_styles.fast_blink));
-                self.fast_blink = new_styles.fast_blink;
             }
+            self.fast_blink = new_styles.fast_blink;
         }
         if self.underline != new_styles.underline {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.underline(new_styles.underline));
-                self.underline = new_styles.underline;
             } else {
                 diff = Some(CharacterStyles::new().underline(new_styles.underline));
-                self.underline = new_styles.underline;
             }
+            self.underline = new_styles.underline;
         }
         if self.bold != new_styles.bold {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.bold(new_styles.bold));
-                self.bold = new_styles.bold;
             } else {
                 diff = Some(CharacterStyles::new().bold(new_styles.bold));
-                self.bold = new_styles.bold;
             }
+            self.bold = new_styles.bold;
         }
         if self.dim != new_styles.dim {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.dim(new_styles.dim));
-                self.dim = new_styles.dim;
             } else {
                 diff = Some(CharacterStyles::new().dim(new_styles.dim));
-                self.dim = new_styles.dim;
             }
+            self.dim = new_styles.dim;
         }
         if self.italic != new_styles.italic {
             if let Some(new_diff) = diff.as_mut() {
                 diff = Some(new_diff.italic(new_styles.italic));
-                self.italic = new_styles.italic;
             } else {
                 diff = Some(CharacterStyles::new().italic(new_styles.italic));
-                self.italic = new_styles.italic;
+            }
+            self.italic = new_styles.italic;
+        }
+
+        if let Some(changed_colors) = changed_colors {
+            if let Some(AnsiCode::ColorIndex(color_index)) = diff.and_then(|diff| diff.foreground) {
+                if let Some(changed_color) = changed_colors[color_index as usize] {
+                    diff.as_mut().unwrap().foreground = Some(changed_color);
+                }
+            }
+            if let Some(AnsiCode::ColorIndex(color_index)) = diff.and_then(|diff| diff.background) {
+                if let Some(changed_color) = changed_colors[color_index as usize] {
+                    diff.as_mut().unwrap().background = Some(changed_color);
+                }
             }
         }
         diff
@@ -448,7 +452,7 @@ impl CharacterStyles {
                     *self = self.background(Some(AnsiCode::NamedColor(NamedColor::BrightWhite)))
                 }
                 _ => {
-                    let _ = debug_log_to_file(format!("unhandled csi m code {:?}", param));
+                    log::warn!("unhandled csi m code {:?}", param);
                     return;
                 }
             }
@@ -708,6 +712,7 @@ impl IndexMut<CharsetIndex> for Charsets {
 
 #[derive(Clone, Copy, Debug)]
 pub enum CursorShape {
+    Initial,
     Block,
     BlinkingBlock,
     Underline,
@@ -734,7 +739,7 @@ impl Cursor {
             is_hidden: false,
             pending_styles: CharacterStyles::new(),
             charsets: Default::default(),
-            shape: CursorShape::Block,
+            shape: CursorShape::Initial,
         }
     }
     pub fn change_shape(&mut self, shape: CursorShape) {
@@ -757,15 +762,3 @@ impl ::std::fmt::Debug for TerminalCharacter {
         write!(f, "{}", self.character)
     }
 }
-
-fn parse_sgr_color(params: &mut dyn Iterator<Item = u16>) -> Option<AnsiCode> {
-    match params.next() {
-        Some(2) => Some(AnsiCode::RgbCode((
-            u8::try_from(params.next()?).ok()?,
-            u8::try_from(params.next()?).ok()?,
-            u8::try_from(params.next()?).ok()?,
-        ))),
-        Some(5) => Some(AnsiCode::ColorIndex(u8::try_from(params.next()?).ok()?)),
-        _ => None,
-    }
-}
