@@ -159,6 +159,17 @@ impl Setup {
             _ => false,
         };
 
+        // setup functions that don't require deserialisation of the config
+        if let Some(Command::Setup(ref setup)) = &opts.command {
+            setup.from_cli().map_or_else(
+                |e| {
+                    eprintln!("{:?}", e);
+                    process::exit(1);
+                },
+                |_| {},
+            );
+        };
+
         let config = if !clean {
             match Config::try_from(opts) {
                 Ok(config) => config,
@@ -190,20 +201,22 @@ impl Setup {
         };
 
         if let Some(Command::Setup(ref setup)) = &opts.command {
-            setup.from_cli(opts, &config_options).map_or_else(
-                |e| {
-                    eprintln!("{:?}", e);
-                    process::exit(1);
-                },
-                |_| {},
-            );
+            setup
+                .from_cli_with_options(opts, &config_options)
+                .map_or_else(
+                    |e| {
+                        eprintln!("{:?}", e);
+                        process::exit(1);
+                    },
+                    |_| {},
+                );
         };
 
         Ok((config, layout, config_options))
     }
 
     /// General setup helpers
-    pub fn from_cli(&self, opts: &CliArgs, config_options: &Options) -> std::io::Result<()> {
+    pub fn from_cli(&self) -> std::io::Result<()> {
         if self.clean {
             return Ok(());
         }
@@ -213,11 +226,6 @@ impl Setup {
             std::process::exit(0);
         }
 
-        if self.check {
-            Setup::check_defaults_config(opts, config_options)?;
-            std::process::exit(0);
-        }
-
         if let Some(shell) = &self.generate_completion {
             Self::generate_completion(shell.into());
             std::process::exit(0);
@@ -231,6 +239,19 @@ impl Setup {
         Ok(())
     }
 
+    /// Checks the merged configuration
+    pub fn from_cli_with_options(
+        &self,
+        opts: &CliArgs,
+        config_options: &Options,
+    ) -> std::io::Result<()> {
+        if self.check {
+            Setup::check_defaults_config(opts, config_options)?;
+            std::process::exit(0);
+        }
+        Ok(())
+    }
+
     pub fn check_defaults_config(opts: &CliArgs, config_options: &Options) -> std::io::Result<()> {
         let data_dir = opts.data_dir.clone().unwrap_or_else(get_default_data_dir);
         let config_dir = opts.config_dir.clone().or_else(find_default_config_dir);
@@ -245,6 +266,12 @@ impl Setup {
             .clone()
             .or_else(|| config_dir.clone().map(|p| p.join(CONFIG_NAME)));
 
+        // according to
+        // https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda
+        let hyperlink_start = "\u{1b}]8;;";
+        let hyperlink_mid = "\u{1b}\\";
+        let hyperlink_end = "\u{1b}]8;;\u{1b}\\\n'";
+
         let mut message = String::new();
 
         message.push_str(&format!("[Version]: {:?}\n", VERSION));
@@ -289,10 +316,26 @@ impl Setup {
         message.push_str(&format!("[ARROW SEPARATOR]: {}\n", ARROW_SEPARATOR));
         message.push_str(" Is the [ARROW_SEPARATOR] displayed correctly?\n");
         message.push_str(" If not you may want to either start zellij with a compatible mode 'zellij options --simplified-ui'\n");
-        message.push_str(" Or check the font that is in use:\n https://zellij.dev/documentation/compatibility.html#the-status-bar-fonts-dont-render-correctly\n");
+        let mut hyperlink_compat = String::new();
+        hyperlink_compat.push_str(hyperlink_start);
+        hyperlink_compat.push_str("https://zellij.dev/documentation/compatibility.html#the-status-bar-fonts-dont-render-correctly");
+        hyperlink_compat.push_str(hyperlink_mid);
+        hyperlink_compat.push_str("https://zellij.dev/documentation/compatibility.html#the-status-bar-fonts-dont-render-correctly");
+        hyperlink_compat.push_str(hyperlink_end);
+        message.push_str(&format!(
+            " Or check the font that is in use:\n {}\n",
+            hyperlink_compat
+        ));
 
         message.push_str(&format!("[FEATURES]: {:?}\n", FEATURES));
-        message.push_str("[DOCUMENTATION]: zellij.dev/documentation/\n");
+        let mut hyperlink = String::new();
+        hyperlink.push_str(hyperlink_start);
+        hyperlink.push_str("https://www.zellij.dev/documentation/");
+        hyperlink.push_str(hyperlink_mid);
+        hyperlink.push_str("zellij.dev/documentation");
+        hyperlink.push_str(hyperlink_end);
+        message.push_str(&format!("[DOCUMENTATION]: {}", hyperlink));
+        //printf '\e]8;;http://example.com\e\\This is a link\e]8;;\e\\\n'
 
         std::io::stdout().write_all(message.as_bytes())?;
 
